@@ -110,15 +110,16 @@ static void  quit_event( UIComponents *uic, guint callback_action, GtkWidget *me
 					
 }
 
-/* callback to check availability of contactinfos service */
+/* callback to poll phone and continue as long as there is polling to do ... */
 gboolean pollPhone_cb(UIComponents *uic) {
 	
 	if (uic->phoneOpened && uic->pollingPhone) {
 		uic->pollingPhone = FALSE;
 		gn_sm_loop(1, state);
 		uic->pollingPhone = TRUE;
+		return TRUE;
 	}
-	return TRUE;
+	return FALSE;
 }
 
 /* callback to check availability of contactinfos service */
@@ -257,8 +258,31 @@ gn_error answer(int lowlevel_id) {
 
 void notify_sms(gn_sms *message, struct gn_statemachine *state, UIComponents *uic) {
 	/* in this callback function you can't use those libgnokii functions that send a packet */
+	gchar *descr;
+	GtkWidget *note;
+	gint resultCode;
 	
-	hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_INFO, _("SMS received from %s :\n%s"), message->remote, message->user_data[0].u.text);
+	g_printf("\n===== ===== Incoming SMS ===== =====\n");
+	g_printf("===== from %s =====\n", message->remote.number);
+
+	descr = g_strdup_printf(_("SMS received from %s :\n%s"), message->remote.number, message->user_data[0].u.text);
+	note = hildon_note_new_confirmation_add_buttons(NULL, descr, _("Reply"), GTK_RESPONSE_ACCEPT, _("Forget it"), GTK_RESPONSE_REJECT, NULL);
+	g_free(descr);
+	gtk_widget_show_all (note);
+
+	resultCode = gtk_dialog_run (GTK_DIALOG (note));
+	
+	switch (resultCode) {
+	case GTK_RESPONSE_ACCEPT:
+		gtk_entry_set_text(GTK_ENTRY(uic->searchEntry), message->remote.number);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(uic->contactInfosCheckBox), FALSE);
+		gtk_container_set_focus_child(GTK_CONTAINER(gtk_widget_get_toplevel(uic->searchEntry)), uic->searchEntry);
+		break;
+	default:
+		break;
+	}
+	gtk_widget_destroy (note);
+
 }
 
 void notify_call(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, UIComponents *uic) {
@@ -327,8 +351,11 @@ gboolean getCalls(GtkWidget *topLevel, UIComponents *uic) {
 			}
 			return FALSE;
 		} else {
+			if (!uic->pollingPhone) {
+				uic->pollingPhone = TRUE;
+				g_idle_add ((GSourceFunc)pollPhone_cb, uic);
+			}
 			hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Waiting for a call ..."));
-			uic->pollingPhone = TRUE;
 		}
 	} else {
 		hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Stopped waiting for a call ..."));
@@ -378,8 +405,11 @@ gboolean getSms(GtkWidget *topLevel, UIComponents *uic) {
 			}
 			return FALSE;
 		} else {
+			if (!uic->pollingPhone) {
+				uic->pollingPhone = TRUE;
+				g_idle_add ((GSourceFunc)pollPhone_cb, uic);
+			}
 			hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Waiting for SMS ..."));
-			uic->pollingPhone = TRUE;
 		}
 	} else {
 		hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Stopped waiting for SMS ..."));
@@ -1220,9 +1250,6 @@ int main( int argc, char *argv[] ) {
 
 	uic->callDetected = -1;
 
-	/* check availability and show or hide the Buttons */
-	g_idle_add ((GSourceFunc)pollPhone_cb, uic);
-
 	if (!uic->noConfig) {
 		if (uic->getSms) {
 			getSms(GTK_WIDGET(appview), uic);
